body {
    color: #73879C;
    background: #F7F7F7;
    font-family: "Helvetica Neue", Roboto, Arial, "Droid Sans", sans-serif;
    font-size: 13px;
    font-weight: 400;
    line-height: 1.471;
}

.main_conainer {
    padding: 10px 20px 0;
}

i span {
    display: none;
}

.tc {
    text-align: center;
}

.fz12 {
    font-size: 12px;
}

.fz20 {
    font-size: 20px;
}

/* Navigation */
nav.navbar {
    background: #EDEDED;
    border-bottom: 1px solid #D9DEE4;
    margin-bottom: 10px;
}

nav .navbar-brand {
    border-right: 1px solid #D9DEE4;
    color: #5A738E;
}

nav .navbar-text {
    font-size: 18px;
    height: 50px;
    margin-bottom: 0;
    margin-top: 0;
    padding: 15px 0;
}

/* Table */
table {
    width: 100%;
}

table.chart tr th:first-of-type {
    width: 50%;
}

.table > thead > tr > th {
    background: #F5F7FA;
}

table.tile h3, table.tile h4, table.tile span {
    font-weight: bold;
    vertical-align: middle !important;
}

table.tile th, table.tile td {
    text-align: center;
}

table.tile th {
    border-bottom: 1px solid #E6ECEE;
}

table.tile td {
    padding: 5px 0;
}

table.tile td ul {
    text-align: left;
    padding-left: 0;
}

table.tile td ul li {
    list-style: none;
    width: 100%;
}

table.tile td ul li a {
    width: 100%;
}

table.tile td ul li a big {
    right: 0;
    float: right;
    margin-right: 13px;
}

table.tile_info {
    width: 100%;
}

table.tile_info td {
    text-align: left;
    padding: 1px;
    font-size: 15px;
}

table.tile_info td p {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    margin: 0;
    line-height: 28px;
}

table.tile_info td i {
    display: inline-block;
    margin-right: 8px;
    font-size: 17px;
    float: left;
    width: 18px;
    line-height: 28px;
    text-align: center;
}

table.tile_info td:first-child {
    width: 65%;
}

td span {
    line-height: 28px;
}

table.tile_info td.percentage {
    text-align: right;

}

/* chart in table */
table td.chart {
    display: inline-block;
    position: relative;
}

table td.chart #feature-chart,
table td.chart #scenario-chart,
table td.chart .total {
    height: 150px;
    margin: 15px auto 0;
    width: 150px;
}

table td.chart .total {
    display: inline-block;
    position: absolute;
    font-size: 2em;
    height: 60px;
    line-height: 60px;
    top: 45px;
    left: 50%;
    -webkit-transform: translateX(-50%);
    transform: translateX(-50%);
    text-align: center;
    vertical-align: middle;
    width: 60px;
}

/* colors */
.ambiguous-color {
    color: #E74C3C !important;
}

.failed-color {
    color: #E74C3C !important;
}

.not-defined-color {
    color: #F39C12 !important;
}

.passed-color {
    color: #1ABB9C !important;
}

.pending-color {
    color: #FFD119 !important;
}

.skipped-color {
    color: #3498DB !important;
}

/* backgrounds */
.ambiguous-background {
    background: #b73122 !important;
}

.failed-background {
    background: #E74C3C !important;
}

.not-defined-background {
    background: #F39C12 !important;
}

.passed-background {
    background: #1ABB9C !important;
}

.pending-background {
    background: #FFD119 !important;
}

.skipped-background {
    background: #3498DB !important;
}

/* general */
.x_panel {
    position: relative;
    width: 100%;
    margin-bottom: 10px;
    padding: 10px 17px;
    display: inline-block;
    background: #fff;
    border: 1px solid #E6E9ED;
    -webkit-column-break-inside: avoid;
    -moz-column-break-inside: avoid;
    column-break-inside: avoid;
    opacity: 1;
    transition: all .2s ease;
}

.x_title {
    border-bottom: 2px solid #E6E9ED;
    padding: 1px 5px 6px;
    margin-bottom: 10px;
}

.x_title .filter {
    width: 40%;
    float: right;
}

.x_title h2 {
    margin: 5px 0 6px;
    float: left;
    font-size: 24px;
    font-weight: 400;
    display: block;
    text-overflow: ellipsis;
    overflow: hidden;
    white-space: nowrap;
}

.fixed_height_320 {
    height: 320px;
}

.x_title h2 small {
    margin-left: 10px;
}

.x_title span {
    color: #BDBDBD;
}

.x_content {
    padding: 0 5px 6px;
    position: relative;
    width: 100%;
    float: left;
    clear: both;
    margin-top: 5px;
}

.x_content h4 {
    font-size: 16px;
    font-weight: 500;
}

.panel_toolbox {
    float: right;
    margin: 5px 0 0;
    min-width: 70px;
}

.panel_toolbox > li {
    float: right;
}

.panel_toolbox > li > a {
    cursor: pointer;
}

.panel_toolbox > li > a {
    padding: 5px;
    color: #C5C7CB;
    font-size: 14px;
}

.panel_toolbox > li > a:hover {
    background: #F5F7FA;
}

.page-title {
    width: 100%;
    padding: 10px 0 30px 0;
}

.page-title {
    display: block;
}

.page-title h1 {
    margin: 9px 0;
}

.page-title .title_right {
    width: 55%;
    float: left;
    display: block;
}

.page-title .title_right .pull-right {
    margin: 10px 0;
}

.page-title p {
    margin-left: 15px;
}

.dashboard-widget-content {
    padding-top: 9px;
}

.dashboard-widget-content .sidebar-widget {
    width: 50%;
    display: inline-block;
    vertical-align: top;
    background: #fff;
    border: 1px solid #abd9ea;
    border-radius: 5px;
    text-align: center;
    float: right;
    padding: 2px;
    margin-top: 10px;
}

ul.quick-list {
    padding-left: 0;
    display: inline-block;
}

ul.quick-list li,
table.quick-list tr {
    padding-left: 10px;
    list-style: none;
    margin: 0;
    padding-bottom: 6px;
    padding-top: 4px;
    white-space: nowrap;
    text-overflow: ellipsis;
    overflow: hidden;
}

ul.quick-list li .meta-data-title,
table.quick-list td.meta-data-title {
    display: inline-block;
    min-width: 75px;
    font-weight: bold;
}

ul.quick-list li span,
table.quick-list td {
    line-height: 28px;
}

ul.quick-list li,
table.quick-list tr {
    border-bottom: 1px solid #efefef;
    padding: 0.5em 0;
}

ul.quick-list li:last-child,
table.quick-list tr:last-child {
    border-bottom: none;
}

ul.quick-list li i {
    padding-right: 10px;
    color: #757679;
}

.screenshot{
    max-height: 100%;
    max-width: 100%;
}

/* Features / Scenarios */
ul.panel_toolbox li .step {
    border-radius: 50%;
    color: #ffffff;
    display: block;
    font-size: 14px;
    height: 30px;
    margin-right: 5px;
    padding: 5px;
    text-align: center;
    width: 30px;
}

.scenario-step-container {
    margin-bottom: 10px;
}

.scenario-step-container .label {
    display: inline-block;
    text-align: center;
    width: 30px;
}

.scenario-step-container .text {
    display: inline;
}

.scenario-step-container .duration {
    position: relative;
    float: right;
}

.scenario-step-container .text .keyword.highlight {
    font-size: 1.2em;
    font-weight: 700;
}

.scenario-scroll-bar {
    overflow-x: scroll;
}

.scenario-step-collapse,
.scenario-scroll-bar .arguments {
    margin-left: 30px;
    width: auto;
}

/* media */
@media (max-width: 1200px) {
    .x_title h2 {
        width: 70%;
    }
}

@media (max-width: 640px) {
    .x_title h2 {
        width: 100%;
    }
}

/* override */
table.dataTable.dtr-inline.collapsed > tbody > tr > td:first-child:before,
table.dataTable.dtr-inline.collapsed > tbody > tr > th:first-child:before {
    background-color: #1ABB9C;
}

table.dataTable.dtr-inline.collapsed > tbody > tr.parent > td:first-child:before, table.dataTable.dtr-inline.collapsed > tbody > tr.parent > th:first-child:before {
    background-color: #E74C3C;
}

.created-by {
    padding: 50px 0;
    text-align: center;
}
